function [numcliques, X, Y, Z, eta, x, Comm, density, srat, chi, entropy, meanden, minden] = mclscpc3(b, adj)
% #######################################################################
% Program mclscpc2 - Maximal Clique Location Set Covering Method - Ver. C 
%       for Overlapping Communities
%       (1) The Bron-Kerbosch algorithm is used to find all maximal cliques.
%       (2) Cliques are assembled into candidate COMMUNITIES based on
%           k-1 adjacency and edge-weight contribution.
%       (3) A set-covering problem is solved with the objective of
%       minimizing the number of COMMUNITIES needed to cover all vertices.
% This program differs from mslscp.m because of Step (2), that is, it is
% comminities that are covering rather than cliques.
% ########################################################################
% INPUTS:
%       b  -  a nonnegative, e-by-e adjacency matrix (e is # of vertices)
%       adj - the specified adjacency (usually 1 or 2)
% OUTPUTS:
%       numcliques - number of maximal cliques of size mink or greater
%       X - an ncliques-by-e matrix, where e is the number of vertices
%           values of 1 indicate vertex in column a member of clique in row
%       Y - an ncomm-by-e matrix, where e is the number of vertices
%           and ncomm is the number of candidate communities
%           values of 1 indicate vertex in column a member of comm in row
%       eta - an ncliques-by-1 vector that indicates the candidate
%             community of the clique.
%       Z - the minimum number of candidate communities needed to cover
%           all vertices
%       x - ILP solution vector with values of 1 indicating the selection
%           of the candidate community
%       Comm - a binary matrix indicating the communities to which each
%              vertex belongs
%       density - a vector of community densities
% #######################################################################
%tic;
a = b > 0;                             % a is a binary version of b
a = double(a);                         % a(i,j) = 1 means an edge present
[n1,e] = size(a); 
[ MC ] = maximalCliques( a, 'v2');
[n2, numcliques] = size(MC);
X = MC';
maxk = max(sum(MC));
mink = min(sum(MC));
if numcliques == 0
    return
end
[iset,m] = size(X);
siz = sum(X,2);
for k = maxk:-1:mink
    XX = X(siz>=k,:);
    sumc = sum(XX,1);
    minsum = min(sumc);
    if minsum == 0
%        name = 'NotAllCovered';
%        disp(name);
    else
        X = XX;
        [iset,m] = size(X);
        break
    end
end

% ##################################################################
% This block computes the sum of the edge weight and size of cliques
% ##################################################################
I = ones(iset,1);         % Sum of the edge weights of the cliques
S = zeros(iset,1);         % Size of the Cliques
for i = 1:iset
    isize = 0;
    for j = 1:m
        if X(i,j) == 1
            isize = isize + 1;
        end
    end
    for j = 1:m-1                       
        for k = j+1:m
            if X(i,j) == 1 && X(i,k) == 1
                I(i) = I(i) .* b(j,k);
            end
        end
    end
    S(i) = isize;
    I(i) = I(i).^(2./(isize.*(isize-1)));
end
% #####################################################################
% This block compputes the symmetric binary matrix 'phi', with elements
% of 1 indicating that the pair of cliques is k-1 adjacent, else 0
% #####################################################################
phi = zeros(iset);           
for i = 1:iset-1
    for j = i+1:iset
        cmin = S(i);
        if S(j) < cmin
            cmin = S(j);
        end
        csum = 0;
        for k = 1:e
            if X(i,k) == 1 && X(j,k) == 1
                csum = csum + 1;
            end
        end
        if csum >= cmin - adj
            phi(i,j) = 1; phi(j,i) = 1;
        end
    end
end
% #####################################################################
% This block combines cliques to form potential communities. This is
% accomplished into tho phases: Phase 1: the pair of k-1 adjacent cliques
% with the largest edge weight sums are merged to form a community. 
% Phase 2: Cliques are added to the community if they are k-1 adjacent to
% everything in the community -- and ties are broken based on largest
% edge weight sums.
% ####################################################################
eta = zeros(iset,1);        % Community Labels
pairs_exist = 1;
icomm = 0;                  % Number of communities
while pairs_exist == 1
    pairs_exist = 0;
    icomm = icomm + 1;
    Imax = 0;
    for i = 1:iset-1
        if eta(i) > 0
            continue
        end
        for j = i+1:iset
            if eta(j) > 0
                continue
            end
            if phi(i,j) == 0
                continue
            end
            if I(i) + I(j) > Imax
                Imax = I(i) + I(j);
                isel = i;
                jsel = j;
                pairs_exist = 1;
            end
        end
    end
    if pairs_exist == 1
        eta(isel) = icomm;
        eta(jsel) = icomm;
        can_add = 1;
        while can_add == 1
            can_add = 0;
            Imax = 0;
            for i = 1:iset        % Is i a candidate to be added to
                if eta(i) > 0     % the current community?
                    continue
                end
                ipass = 0;
                for k = 1:iset
                    if k == i
                        continue
                    end
                    if eta(k) == icomm && phi(i,k) == 0
                        ipass = 1;
                        continue
                    end
                end
                if ipass == 0
                    if I(i) > Imax
                        Imax = I(i);
                        isel = i;
                        can_add = 1;
                    end
                end
            end
            if can_add == 1
                eta(isel) = icomm;
            end
        end
    end
end
% #####################################################################
% In this block we form the community coverage matrix from the clique
% coverage matrix. Specifically, the elements of the community coverage
% matrix Y are 1 if the community covers the vertex.
% #####################################################################
icomm = icomm - 1;
Y = zeros(icomm,m);
for k = 1:icomm
    for i = 1:iset
        if eta(i) ~= k
            continue
        end
        for j = 1:m
            if X(i,j) > 0.9
                Y(k,j) = 1;
            end
        end
    end
end
for i = 1:iset
    if eta(i) > 0
        continue
    end
    Y = [Y; X(i,:)];
end
[t1,t2] = size(Y);
iset = t1;
        
%iperm = randperm(iset);        % Permute the cliques
% ######################################################
% Find the minimum number of cliques needed to cover all
% vertices using Integer Linear Programming solution of
% a set-covering problem.
% ######################################################
f = ones(iset,1);             % Decision variables
cons = m;
bb = -ones(cons,1);
A = zeros(cons,iset);
icon = 0;
% ############################################################
% Constraint Set 1: There is a covering constraint for each vertex
% ############################################################
for i = 1:cons
    for j = 1:iset
        if Y(j,i) > 0.9
            A(i,j) = -1;
        end
    end
end
intcon = 1:iset;
%size(f)
%size(A)
lb = zeros(1,length(f));       % Variable lower bounds
ub = ones(1,length(f));        % Variable upper bounds
%options = optimoptions("intlinprog",'Heuristics','advanced');
options = optimoptions("intlinprog",'Heuristics','advanced','Display','none');
%options = optimoptions('intlinprog','Display','none');
[x, Z] = intlinprog(f,intcon,A,bb,[],[],lb,ub,options);
Comm = Y(x>0,:)';
Z = round(Z);
density = zeros(Z,1);
a = b>0;
a = double(a);
for i = 1:m
    a(i,i) = 0;
end
for i = 1:Z
    u = find(Comm(:,i)==1);
    usize = length(u);
    U = a(u,u);
    usum = sum(sum(U))./2;
    density(i) = usum./(usize.*(usize-1)./2);
end
%commsizes = sum(Comm);
%memcon = 1./sum(Comm,2);
%q = sort(commsizes);
%srat = q(Z)./q(Z-1);       % Ratio of largest to second largest community
%if Z == 2
%    chi = 0;
%else
%   chi = 0;
%    for i = 1:Z-1
%        num = q(i).^2;
%        denom = 0;
%        for j = 1:Z-1
%            if j ~= i
%                denom = denom + q(j);
%            end
%        end
%        denom = denom.^2;
%        chi = chi + num./denom;
%    end
% end
chi = 0;
entropy = 0;
srat = 0;
%denom = m;    % sum(commsizes);
%for j = 1:Z
%    num = 0;
%    for i = 1:m
%        if Comm(i,j) > 0.9
%            num = num + memcon(i);
%        end
%    end
%    entropy = entropy + (num./denom).*log2(num/denom);
% end
% entropy = - entropy;
meanden = mean(density);
minden = min(density);
%toc